class COM_CONNECTION:
    def __init__(self, data: dict):
        self.METHOD = "COM"
        self.PORT = data["PORT"]
        self.BAUDRATE = data["BAUDRATE"]
        self.DEVICE = data["DEVICE"]+"_serial"
        self.EXECPASS = data["EXECPASS"] if data.get("EXECPASS") else ""

    def __str__(self):
        print(self.METHOD, self.PORT, self.BAUDRATE, self.DEVICE, self.EXECPASS)

    def getNetmikoConnDict(self):
        return {
            'device_type': self.DEVICE,
            'serial_settings': {
                'port': self.PORT,
                'baudrate': self.BAUDRATE,
            },
            'username': '',
            'password': '',
            'secret': self.EXECPASS,
        }

class SSH_CONNECTION:
    def __init__(self, data: dict):
        self.METHOD = data["METHOD"]
        self.HOST = data["HOST"]
        self.PORT = data["PORT"]
        self.USERNAME = data["USERNAME"]
        self.PASSWORD = data["PASSWORD"]
        self.DEVICE = data["DEVICE"]
        self.EXECPASS = data["EXECPASS"] if data.get("EXECPASS") else ""

    def __str__(self):
        print(self.METHOD, self.HOST, self.PORT, self.USERNAME, self.PASSWORD, self.EXECPASS)

    def getNetmikoConnDict(self):
        return {
            'device_type': self.DEVICE,
            'host': self.HOST,
            'port': self.PORT,
            'username': self.USERNAME,
            'password': self.PASSWORD,
            'secret': self.EXECPASS,
        }

class TELNET_CONNECTION:
    def __init__(self, data: dict):
        self.METHOD = data["METHOD"]
        self.HOST = data["HOST"]
        self.PORT = data["PORT"]
        self.PASSWORD = data["PASSWORD"]
        self.DEVICE = data["DEVICE"] + "_telnet"
        self.EXECPASS = data["EXECPASS"] if data.get("EXECPASS") else ""

    def __str__(self):
        print(self.METHOD, self.HOST, self.PORT, self.PASSWORD, self.EXECPASS)

    def getNetmikoConnDict(self):
        return {
            'device_type': self.DEVICE,
            'host': self.HOST,
            'port': self.PORT,
            'password': self.PASSWORD,
            'secret': self.EXECPASS,
        }

class TFTP_CONNECTION:
    pass

def read_nmconn(file: str) -> COM_CONNECTION | SSH_CONNECTION | TELNET_CONNECTION | TFTP_CONNECTION:
    with open(file, 'r') as f:
        lines = f.readlines()
        conn_data = {}
        for idx, line in enumerate(lines):
            line = line.strip()
            if len(line) > 0 and line[0:2] != '--':
                key, value = line.split(': ')
                print(key, value)
                conn_data[key] = value
        print(conn_data)

        if conn_data.get("DEVICE") is None:
            raise ValueError("DEVICE is not defined")

        if conn_data.get("METHOD") == "COM":
            if conn_data.get("PORT") is None or conn_data.get("BAUDRATE") is None:
                raise AttributeError("Expected more data.")
            return COM_CONNECTION(conn_data)
        elif conn_data.get("METHOD") == "SSH":
            if conn_data.get("HOST") is None or conn_data.get("PORT") is None or conn_data.get("USERNAME") or conn_data.get("PASSWORD") is None:
                raise AttributeError("Expected more data.")
            return SSH_CONNECTION(conn_data)

        elif conn_data.get("METHOD") == "TELNET":
            if conn_data.get("HOST") is None or conn_data.get("PORT") is None or conn_data.get("PASSWORD") is None:
                raise AttributeError("Expected more data.")
            return TELNET_CONNECTION(conn_data)

        elif conn_data.get("METHOD") == "TFTP":
            return TFTP_CONNECTION()
        else:
            raise AttributeError("Unknown or unhandled connection type.")

def create_nmconn(name, output, method, device, ip, port, username, password, exec, baudrate):
    if output is not None:
        with open(output, 'w+', encoding="UTF-8") as f:
            f.writelines([
                "-- META\n",
                f"NAME: {name}\n",
                f"METHOD: {method}\n",
            ])

            if method == "COM":
                f.writelines([
                    "-- DATA\n",
                    f"PORT: {port}\n",
                    f"BAUDRATE: {baudrate}\n",
                    f"EXECPASS: {exec}\n",
                ])
            elif method == "SSH":
                f.writelines([
                    "-- DATA\n",
                    f"HOST: {ip}\n",
                    f"PORT: {port}\n",
                    f"USERNAME: {username}\n",
                    f"PASSWORD: {password}\n",
                    f"EXECPASS: {exec}\n",
                ])
            elif method == "TELNET":
                f.writelines([
                    "-- DATA\n",
                    f"HOST: {ip}\n",
                    f"PORT: {port}\n",
                    f"PASSWORD: {password}\n",
                    f"EXECPASS: {exec}\n",
                ])
            f.writelines([
                "-- DEVICE\n",
                f"DEVICE: {device}\n"
            ])

if __name__ == '__main__':
    read_nmconn("switch_com.nmconn")


import argparse
from netmiko import ConnectHandler, NetmikoTimeoutException, NetmikoAuthenticationException
from utils import SSH_CONNECTION, TELNET_CONNECTION, COM_CONNECTION, TFTP_CONNECTION, read_nmconn, create_nmconn


def read_config(connection: str, output_file: str | None, show_config: bool | None):
    connection = read_nmconn(connection)

    if connection.METHOD != "TFTP":
        connection_data = connection.getNetmikoConnDict()

        net_connect = ConnectHandler(**connection_data)

        if (connection_data['device_type'].startswith('cisco_') or
            connection_data['device_type'].startswith('juniper_') or
            connection_data['device_type'].startswith('fortinet_')) and \
                connection_data['secret']:
            net_connect.enable()

        if connection_data['device_type'].startswith('cisco_'):
            command = 'show running-config'
        elif connection_data['device_type'].startswith('juniper_'):
            command = 'show configuration'
        elif connection_data['device_type'].startswith('hp_'):
            command = 'display current-configuration'
        elif connection_data['device_type'].startswith('fortinet_'):
            command = 'show'
        elif connection_data['device_type'].startswith('ubiquiti_'):
            command = 'show configuration'
        elif connection_data['device_type'].startswith('mikrotik_'):
            command = '/export'
        elif connection_data['device_type'].startswith('paloalto_'):
            command = 'show config running'
        else:
            raise ValueError("Unsupported device type")

        output = net_connect.send_command(command)
        if show_config:
            print(output)

        if output is not None:
            with open(output_file, 'w+') as f:
                f.write(output)

        net_connect.disconnect()

def create_connection(name, output, method, device, ip, port, username, password, exec, baudrate):
    try:
        if not((method == "COM" and port is not None and baudrate is not None) or ((method == "SSH" or method == "TELNET") and ip is not None and port is not None and (username is not None or method == "TELNET") and password is not None)):
            raise AttributeError("Method must be either COM or SSH or TELNET and certain data need to be provided.")
        create_nmconn(name, output, method, device, ip, port, username, password, exec, baudrate)
        print("Success")
    except Exception as e:
        print(e)
        print("Fail")

def test_connection(connectionFile):
    conn = read_nmconn(connectionFile)
    connection_data = conn.getNetmikoConnDict()

    try:
        connection = ConnectHandler(**connection_data)
        print("Success")
        connection.disconnect()

    except NetmikoTimeoutException:
        print("Error: request timed out")

    except NetmikoAuthenticationException:
        print("Error: wrong login or password")

    except Exception as e:
        print(f"Error: {e}")

def main():
    parser = argparse.ArgumentParser(description='NetManage')
    subparsers = parser.add_subparsers(dest='command')

    # Read conf
    parser_read_config = subparsers.add_parser('read-config', help='Read config file')
    parser_read_config.add_argument('-c', '--connection', type=str, required=True, help='Path to .nmconn file')
    parser_read_config.add_argument('-o', '--output', type=str, required=False, help='Path to output .txt file')
    parser_read_config.add_argument('-s', '--show-config', type=bool, required=False, help='Show output in console')

    # create .nmconn file

    parser_create_connection = subparsers.add_parser('create-conn', help='Create connection')
    parser_create_connection.add_argument('-n', '--name', type=str, required=True, help='Connection name')
    parser_create_connection.add_argument('-o', '--output', type=str, required=True, help='Path to .nmconn file')
    parser_create_connection.add_argument('-m', '--method', type=str, required=True, help='Method: SSH, TELNET, COM, TFTP')
    parser_create_connection.add_argument('-d', '--device', type=str, required=True, help='Device_type z bilbioteki netmiko')
    parser_create_connection.add_argument('-i', '--ip', type=str, required=False, help='[SSH/TELNET] Host')
    parser_create_connection.add_argument('-po', '--port', type=str, required=False, help='[SSH/TELNET/COM] PORT')
    parser_create_connection.add_argument('-b', '--baudrate', type=int, required=False, help='[COM] Baudrate')
    parser_create_connection.add_argument('-u', '--username', type=str, required=False, help='[SSH/TELNET] Username')
    parser_create_connection.add_argument('-pa', '--password', type=str, required=False, help='[SSH/TELNET] Password')
    parser_create_connection.add_argument('-e', '--exec', type=str, required=False, help='[SSH/TELNET/COM] EXEC')

    # test connection

    parser_test_connection = subparsers.add_parser('test-conn', help='Test connection')
    parser_test_connection.add_argument('-c', '--connection', type=str, required=True, help='Path to .nmconn file')

    args = parser.parse_args()

    if args.command == "read-config":
        print(
            args.connection, args.output, args.show_config
        )

        read_config(args.connection, args.output, args.show_config)
    elif args.command == "create-conn":
        print(
            args.name, args.output, args.method, args.device, args.ip, args.port, args.username, args.password, args.exec, args.baudrate
        )

        create_connection(args.name, args.output, args.method, args.device, args.ip, args.port, args.username, args.password, args.exec, args.baudrate)
    elif args.command == "test-conn":
        test_connection(args.connection)
    else:
        parser.print_help()

if __name__ == '__main__':
    main()
